package com.utopiaindustries.qualitychecker.ui.activities;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.utopiaindustries.qualitychecker.R;
import com.utopiaindustries.qualitychecker.apiservice.ApiService;
import com.utopiaindustries.qualitychecker.apiservice.ApiServiceFactory;
import com.utopiaindustries.qualitychecker.db.CheckpointRepository;
import com.utopiaindustries.qualitychecker.db.DefectRepository;
import com.utopiaindustries.qualitychecker.db.DimensionRepository;
import com.utopiaindustries.qualitychecker.db.ItemRepository;
import com.utopiaindustries.qualitychecker.db.ProductRepository;
import com.utopiaindustries.qualitychecker.models.EmployeePhoto;
import com.utopiaindustries.qualitychecker.models.InspectionReport;
import com.utopiaindustries.qualitychecker.models.callback.SaveCheckpointCallback;
import com.utopiaindustries.qualitychecker.models.callback.SaveDefectCallback;
import com.utopiaindustries.qualitychecker.models.callback.SaveDimensionCallback;
import com.utopiaindustries.qualitychecker.models.callback.SaveItemCallback;
import com.utopiaindustries.qualitychecker.models.callback.SaveProductCallBack;
import com.utopiaindustries.qualitychecker.notification.NotificationHelper;
import com.utopiaindustries.qualitychecker.receiver.NetworkReceiver;
import com.utopiaindustries.qualitychecker.service.InspectionReportService;
import com.utopiaindustries.qualitychecker.store.Store;
import com.utopiaindustries.qualitychecker.ui.adapter.ReportAdapter;

import java.util.List;
import java.util.Objects;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class HomeActivity extends AppCompatActivity implements View.OnClickListener {

    private Button createReportBtn,refreshReportsBtn,logoutBtn,fetchProductBtn;
    private RecyclerView recyclerView;
    private ApiService apiService;
    private TextView usernameTitle, emptyReportTextView;
    private ImageView profileImage;
    private Store store;
    private InspectionReportService inspectionReportService;
    private NetworkReceiver networkReceiver;
    private boolean isReceiverRegistered = false;

    @SuppressLint("SourceLockedOrientationActivity")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        if( ! isNetworkConnected() ){
            Toast.makeText( this, "No Internet Connection", Toast.LENGTH_LONG ).show();
        }
        setContentView(R.layout.activity_home);
        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });
        setRequestedOrientation( ActivityInfo.SCREEN_ORIENTATION_PORTRAIT );

        // initialize views
        store = Store.getInstance();
        apiService = ApiServiceFactory.getApiService();
        inspectionReportService = InspectionReportService.getInstance();
        createReportBtn = findViewById( R.id.create_report_btn );
        refreshReportsBtn = findViewById( R.id.refresh_btn );
        logoutBtn = findViewById( R.id.logout_btn );
        fetchProductBtn = findViewById( R.id.fetch_product_btn );
        recyclerView = findViewById( R.id.reports_recyclerview );
        usernameTitle = findViewById( R.id.username_title );
        profileImage = findViewById( R.id.profile_image );
        emptyReportTextView = findViewById( R.id.empty_report_text );

        refreshReportsBtn.setOnClickListener( this );
        createReportBtn.setOnClickListener( this );
        logoutBtn.setOnClickListener( this );
        fetchProductBtn.setOnClickListener( this );

        recyclerView.setLayoutManager( new LinearLayoutManager( this ) );

        updateProfileImage();
        refreshReportData();
    }

    private void refreshReportData(){
        SharedPreferences sharedPreferences = getSharedPreferences("login_prefs", Context.MODE_PRIVATE);
        String username = sharedPreferences.getString("username", null );
        apiService.fetchAllReports( username ).enqueue(
                new Callback<List<InspectionReport>>() {
                    @Override
                    public void onResponse(Call<List<InspectionReport>> call, Response<List<InspectionReport>> response) {
                        if( response.isSuccessful() && response.body() != null ){
                            System.out.println( response.body() );
                            ReportAdapter adapter = new ReportAdapter( response.body() );
                            recyclerView.setAdapter( adapter );
                            if( response.body().isEmpty() )
                                emptyReportTextView.setVisibility( View.VISIBLE ) ;
                        } else {
                            System.out.println("Error");
                            emptyReportTextView.setVisibility( View.VISIBLE ) ;
                        }
                    }
                    @Override
                    public void onFailure(Call<List<InspectionReport>> call, Throwable t) {
                        System.out.println( t.getMessage() );
                        emptyReportTextView.setVisibility( View.VISIBLE ) ;
                    }
                }
        );
    }

    @Override
    public void onClick(View v) {
        int id = v.getId();
        if( id == R.id.create_report_btn ){
            Intent intent = new Intent( this, MainActivity.class );
            startActivity( intent );
        }
        if( id == R.id.logout_btn ){
            SharedPreferences sharedPreferences = getSharedPreferences("login_prefs", Context.MODE_PRIVATE);
            SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putString("username", null );
            editor.putString("password", null );
            editor.apply();
            Intent intent = new Intent( HomeActivity.this, LoginActivity.class );
            startActivity( intent );
            finish();
        }
        if( id == R.id.refresh_btn ){
            refreshReportData();
            updateProfileImage();
        }
        if( id == R.id.fetch_product_btn ){
            if( isNetworkConnected() ) {
                fetchAndPopulateProductData();
                fetchAndPopulateDimensionsData();
                fetchAndPopulateCheckpointData();
                fetchAndPopulateDefectsData();
                fetchAndPopulateUnitsData();
            } else {
                Toast.makeText( this, "network not available", Toast.LENGTH_LONG ).show();
            }
        }
    }

    private void updateProfileImage(){
        SharedPreferences sharedPreferences = getSharedPreferences("login_prefs", Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        String username = sharedPreferences.getString("username", null );
        if( username != null ){
            apiService.fetchEmployeePhoto( username ).enqueue(
                    new Callback<EmployeePhoto>() {
                        @Override
                        public void onResponse(Call<EmployeePhoto> call, Response<EmployeePhoto> response) {
                            System.out.println( response.body() );
                            if( response.isSuccessful() && response.body() != null ){
                                System.out.println( response.body() );
                                editor.putString("name" , response.body().getName() );
                                editor.apply();
                                usernameTitle.setText( response.body().getName() );
                                store.setEmployeePhoto( response.body() );
                                Glide.with( getBaseContext() )
                                        .load( store.getProfileImage( ) )
                                        .into( profileImage );
                            }
                        }

                        @Override
                        public void onFailure(Call<EmployeePhoto> call, Throwable t) {
                            System.out.println( t.getMessage() );
                            String name = sharedPreferences.getString("name", null);
                            usernameTitle.setText( name );
                        }
                    }
            );

        }
    }


    /*
    *  fetch and update product map
    * */
    private void fetchAndPopulateProductData(){

        NotificationHelper.showProductNotification(
                this,
                "Utopia QA App",
                "Fetching Product Details.");
        inspectionReportService.fetchAndPopulateProducts( new SaveProductCallBack() {
            @Override
            public void onSuccess() {
                Toast.makeText( HomeActivity.this, "Products successfully synced", Toast.LENGTH_LONG ).show();
                NotificationHelper.showProductNotification(
                        HomeActivity.this,
                        "Utopia QA App",
                        "Products successfully synced");
            }

            @Override
            public void onFailure(Throwable throwable) {
                Toast.makeText( HomeActivity.this, "Error in Products syncing " + throwable.getMessage(), Toast.LENGTH_LONG ).show();
                NotificationHelper.showProductNotification(
                        HomeActivity.this,
                        "Utopia QA App",
                        "Error in Products syncing" );
            }
        }, new ProductRepository( this )  );
    }

    private void fetchAndPopulateDimensionsData(){

        NotificationHelper.showProductNotification( this,
                "Utopia QA App",
                "Fetching Dimensions Details.");
        inspectionReportService.fetchAndPopulateDimensions( new SaveDimensionCallback() {
            @Override
            public void onSuccess() {
                Toast.makeText( HomeActivity.this, "Dimensions successfully synced", Toast.LENGTH_LONG ).show();
                NotificationHelper.showProductNotification(
                        HomeActivity.this,
                        "Utopia QA App",
                        "Dimensions successfully synced");
            }

            @Override
            public void onFailure(Throwable throwable) {
                Toast.makeText( HomeActivity.this, "Error in Dimensions syncing " + throwable.getMessage(), Toast.LENGTH_LONG ).show();
                NotificationHelper.showProductNotification(
                        HomeActivity.this,
                        "Utopia QA App",
                        "Error in Dimensions syncing" );
            }
        }, new DimensionRepository( this )  );
    }

    private void fetchAndPopulateCheckpointData(){

        NotificationHelper.showProductNotification(
                this,
                "Utopia QA App",
                "Fetching Checkpoints Details.");
        inspectionReportService.fetchAndPopulateCheckpoints( new SaveCheckpointCallback() {
            @Override
            public void onSuccess() {
                Toast.makeText( HomeActivity.this, "Checkpoints successfully synced", Toast.LENGTH_LONG ).show();
                NotificationHelper.showProductNotification( HomeActivity.this,
                        "Utopia QA App",
                        "Checkpoints successfully synced");
            }

            @Override
            public void onFailure(Throwable throwable) {
                Toast.makeText( HomeActivity.this, "Error in Checkpoints syncing " + throwable.getMessage(), Toast.LENGTH_LONG ).show();
                NotificationHelper.showProductNotification(
                        HomeActivity.this,
                        "Utopia QA App",
                        "Error in Checkpoints syncing" );
            }
        }, new CheckpointRepository( this )  );
    }

    private void fetchAndPopulateDefectsData(){

        NotificationHelper.showProductNotification(
                this,
                "Utopia QA App",
                "Fetching Defects Details.");
        inspectionReportService.fetchAndPopulateDefects( new SaveDefectCallback() {
            @Override
            public void onSuccess() {
                Toast.makeText( HomeActivity.this, "Defects successfully synced", Toast.LENGTH_LONG ).show();
                NotificationHelper.showProductNotification(
                        HomeActivity.this,
                        "Utopia QA App",
                        "Defects successfully synced");
            }

            @Override
            public void onFailure(Throwable throwable) {
                Toast.makeText( HomeActivity.this, "Error in Defects syncing " + throwable.getMessage(), Toast.LENGTH_LONG ).show();
                NotificationHelper.showProductNotification(
                        HomeActivity.this,
                        "Utopia QA App",
                        "Error in Defects syncing" );
            }
        }, new DefectRepository( this )  );
    }

    private void fetchAndPopulateUnitsData(){

        NotificationHelper.showProductNotification(
                this,
                "Utopia QA App",
                "Fetching Units Details.");
        inspectionReportService.fetchAndPopulateUnits( new SaveItemCallback() {
            @Override
            public void onSuccess() {
                Toast.makeText( HomeActivity.this, "Units successfully synced", Toast.LENGTH_LONG ).show();
                NotificationHelper.showProductNotification(
                        HomeActivity.this,
                        "Utopia QA App",
                        "Units successfully synced");
            }

            @Override
            public void onFailure(Throwable throwable) {
                Toast.makeText( HomeActivity.this, "Error in Units syncing " + throwable.getMessage(), Toast.LENGTH_LONG ).show();
                NotificationHelper.showProductNotification(
                        HomeActivity.this,
                        "Utopia QA App",
                        "Error in Units syncing" );
            }
        }, new ItemRepository( this )  );
    }



    private boolean isNetworkConnected() {
        ConnectivityManager connectivityManager = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetwork = connectivityManager.getActiveNetworkInfo();
        return activeNetwork != null && activeNetwork.isConnectedOrConnecting();
    }

    @Override
    protected void onResume() {
        super.onResume();
        refreshReportData();
    }

    @Override
    protected void onStart() {
        super.onStart();
        if (networkReceiver == null) {
            networkReceiver = new NetworkReceiver();
        }
        IntentFilter filter = new IntentFilter(ConnectivityManager.CONNECTIVITY_ACTION);
        registerReceiver(networkReceiver, filter);
        isReceiverRegistered = true;  // Mark the receiver as registered
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (isReceiverRegistered) {
            unregisterReceiver(networkReceiver);
            isReceiverRegistered = false;  // Mark the receiver as unregistered
        }
    }
}