package com.utopiaindustries.qualitychecker.ui.fragments;

import static android.app.Activity.RESULT_OK;

import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Bundle;
import android.util.Base64;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.utopiaindustries.qualitychecker.R;
import com.utopiaindustries.qualitychecker.apiservice.ApiService;
import com.utopiaindustries.qualitychecker.apiservice.ApiServiceFactory;
import com.utopiaindustries.qualitychecker.async.ImageProcessor;
import com.utopiaindustries.qualitychecker.models.InspectionCheckPoint;
import com.utopiaindustries.qualitychecker.models.InspectionDefect;
import com.utopiaindustries.qualitychecker.service.InspectionReportService;
import com.utopiaindustries.qualitychecker.store.Store;
import com.utopiaindustries.qualitychecker.ui.adapter.CheckPointAdapter;


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Future;


public class SecondStepFragment extends Fragment
        implements View.OnClickListener {

    private NavController navController;
    private Button leftFab,rightFab;
    private RecyclerView recyclerView;
    private CheckPointAdapter adapter;
    private ApiService apiService;
    private TextView profileName;
    private ImageView profileImage;
    private List<InspectionCheckPoint> checkPoints;
    private List<InspectionDefect> defects;
    private InspectionReportService inspectionReportService;
    private Store store;
    private static final int MY_PERMISSIONS_REQUEST_READ_EXTERNAL_STORAGE = 100;
    private static final int  MY_PERMISSIONS_REQUEST_WRITE_EXTERNAL_STORAGE = 101;


    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_second_step , container, false );

        apiService = ApiServiceFactory.getApiService();
        store = Store.getInstance();
        inspectionReportService = InspectionReportService.getInstance();
        checkPoints = new ArrayList<>();
        defects = new ArrayList<>();
        leftFab = view.findViewById( R.id.button_left_frag_2 );
        rightFab = view.findViewById( R.id.button_right_frag_2 );
        recyclerView  = view.findViewById( R.id.check_point_recycler_view );
        profileName = view.findViewById( R.id.second_profile_name );
        profileImage = view.findViewById( R.id.second_step_profile_image );
        recyclerView.setLayoutManager( new LinearLayoutManager( getContext() ) );

        // PERMISSIONS
        {
            // Check if permission is not granted
            if ( ContextCompat.checkSelfPermission( getContext(), Manifest.permission.READ_EXTERNAL_STORAGE )
                    != PackageManager.PERMISSION_GRANTED) {

                // Permission is not granted, request it
                ActivityCompat.requestPermissions( getActivity(),
                        new String[]{Manifest.permission.READ_EXTERNAL_STORAGE},
                        MY_PERMISSIONS_REQUEST_READ_EXTERNAL_STORAGE);
            }

            if (ContextCompat.checkSelfPermission( getContext(), Manifest.permission.WRITE_EXTERNAL_STORAGE)
                    != PackageManager.PERMISSION_GRANTED) {

                ActivityCompat.requestPermissions( getActivity(),
                        new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
                        MY_PERMISSIONS_REQUEST_WRITE_EXTERNAL_STORAGE);
            }
        }

        adapter = new CheckPointAdapter( store.getReport().getItems().get( 0 ).getCheckPoints(),
                    store.getCheckPoints(),
                    store.getDefects(),
                    inspectionReportService.getDefectTypes(),
                    getContext());
                recyclerView.setAdapter( adapter );
        // set on click listener
        leftFab.setOnClickListener( this );
        rightFab.setOnClickListener( this );
        updateProfileViews();
        return view;
    }

    @Override
    public void onClick(View v) {
        navController = Navigation.findNavController( requireView() );
        if( v.getId() == R.id.button_left_frag_2 ){
            navController.navigate( R.id.action_secondStepFragment_to_firstStepFragment );
        }
        if( v.getId() == R.id.button_right_frag_2 ){
            checkIsRecyclerViewIsCompletelyScrolled( recyclerView );
        }
    }

    public void onImagePickerResult(int requestCode, int resultCode, Intent data) {
        if ( resultCode == RESULT_OK  && data != null ) {
            try {
                ImageProcessor imageProcessor = new ImageProcessor();
                Uri selectedImageUri = data.getData();
                assert selectedImageUri != null;
                String imagePath = selectedImageUri.getPath();
                store.getReport().getItems().get(0).getCheckPoints().get(requestCode).setImagePath(imagePath);

                imageProcessor.compressImageFromUriAsync( getContext(), selectedImageUri, 200, compressedData -> {
                    if (compressedData != null) {
                        store.getReport().getItems().get(0).getCheckPoints().get(requestCode).setFile(compressedData);
                    } else {
                        store.getReport().getItems().get(0).getCheckPoints().get(requestCode).setFile(null);
                    }
                    adapter.notifyItemChanged( requestCode );
                });
            } catch ( Exception e ){
                e.printStackTrace();
            }
        }
    }

    private void checkIsRecyclerViewIsCompletelyScrolled( RecyclerView recyclerView ){
        if (recyclerView.getLayoutManager() instanceof LinearLayoutManager) {
            LinearLayoutManager layoutManager = (LinearLayoutManager) recyclerView.getLayoutManager();
            int lastVisibleItemPosition = layoutManager.findLastCompletelyVisibleItemPosition();
            int totalItemCount = layoutManager.getItemCount();

            if (lastVisibleItemPosition == totalItemCount - 1) {
                navController.navigate( R.id.action_secondStepFragment_to_thirdStepFragment );
            } else {
                Toast.makeText( getContext(), "Please Scroll at the bottom to continue", Toast.LENGTH_SHORT ).show();
            }
        }
    }

    private void updateProfileViews( ) throws NullPointerException{
        SharedPreferences sharedPreferences = getActivity().getSharedPreferences("login_prefs", Context.MODE_PRIVATE);
        profileName.setText( sharedPreferences.getString("name", null ) );
        Glide.with( getContext() )
                .load( store.getProfileImage( ) )
                .into( profileImage );
    }
}
